'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (op) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  if (!pooledProc) pooledProc = op.procPool.prepare(eventCompiler);

  ++lastInstance;
  pooledProc.all({ createInstance: true, instance: lastInstance, opts: opts });

  return (0, _stream.mapEvents)(op.stream, adaptEvent(lastInstance, pooledProc));
};

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _sighCore = require('sigh-core');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _stream = require('sigh-core/lib/stream');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function eventCompiler() {
  var babel = require('babel-core');
  var _ = require('lodash');
  var instances = {};

  return function (instance, event) {
    if (instance.createInstance) {
      var newInstance = instance.instance;
      instances[newInstance] = instance.opts;
      return {};
    }
    var opts = instances[instance];

    var projectPath = event.projectPath,
        path = event.path;


    var babelOpts = _.assign({}, {
      filename: path,
      filenameRelative: projectPath,
      moduleId: projectPath.replace(/\..*$/, ''),
      moduleRoot: event.basePath,
      sourceFileName: path,
      sourceMapTarget: path,
      sourceMap: true
    }, opts);

    var result = babel.transform(event.data, babelOpts);
    return _.pick(result, 'code', 'map');
  };
}

// (de)serialise argument to and result of babel subprocess
function adaptEvent(instance, compiler) {
  return function (event) {
    if (event.type !== 'add' && event.type !== 'change') return event;

    var fileType = event.fileType;

    if (fileType !== 'js' && fileType !== 'es6') return event;

    var result = compiler(instance, _lodash2.default.pick(event, 'type', 'data', 'path', 'basePath', 'projectPath'));

    // without proc pool a Promise.resolve is needed here
    return result.then(function (result) {
      event.data = result.code;
      event.applySourceMap(result.map);
      return event;
    });
  };
}

var pooledProc;
var lastInstance = 0;
//# sourceMappingURL=index.js.map